@@ -13,12 +13,10 @@ from nipype.interfaces.base import BaseInterface, TraitedSpec, File, traits
 
 class DesignMatrixInputSpec(TraitedSpec):
     bold_file = File(exists=True, mandatory=True)
-    session_info = traits.Dict()
-    drop_missing = traits.Bool(
-        desc='Drop columns in design matrix with all missing values')
+    design_info = traits.Dict()
+    drop_missing = traits.Bool(desc='Drop columns in design matrix with all missing values')
     drift_model = traits.Either(
-        traits.String(), None,
-        desc='Optional drift model to apply to design matrix'
+        traits.String(), None, desc='Optional drift model to apply to design matrix'
     )
 
 
@@ -35,7 +33,7 @@ class FirstLevelEstimatorInputSpec(TraitedSpec):
     bold_file = File(exists=True, mandatory=True)
     mask_file = traits.Either(File(exists=True), None)
     design_matrix = File(exists=True, mandatory=True)
-    contrast_info = traits.List(traits.Dict)
+    spec = traits.Dict()
     smoothing_fwhm = traits.Float(desc='Full-width half max (FWHM) in mm for smoothing in mask')
     smoothing_type = traits.Enum('iso', 'isoblurto', desc='Type of smoothing (iso or isoblurto)')
 
@@ -60,7 +58,7 @@ class SecondLevelEstimatorInputSpec(TraitedSpec):
     effect_maps = traits.List(traits.List(File(exists=True)), mandatory=True)
     variance_maps = traits.List(traits.List(File(exists=True)))
     stat_metadata = traits.List(traits.List(traits.Dict), mandatory=True)
-    contrast_info = traits.List(traits.Dict, mandatory=True)
+    spec = traits.Dict()
     smoothing_fwhm = traits.Float(desc='Full-width half max (FWHM) in mm for smoothing in mask')
     smoothing_type = traits.Enum('iso', 'isoblurto', desc='Type of smoothing (iso or isoblurto)')
 
