@@ -1,25 +1,24 @@
 # coding: utf-8
-from pathlib import Path
+import io
 import os
 import os.path as op
-import numpy as np
-import pandas as pd
-import io
 import sys
 import xml.etree.ElementTree as ET
-import nibabel as nb
+from pathlib import Path
 
+import nibabel as nb
+import numpy as np
+import pandas as pd
 from nipype.interfaces.afni.base import (
     AFNICommand,
     AFNICommandInputSpec,
     AFNICommandOutputSpec,
     Info,
 )
-from nipype.interfaces.base import traits, isdefined, File
+from nipype.interfaces.base import File, isdefined, traits
 from nipype.utils.filemanip import fname_presuffix
 
-from .nistats import FirstLevelModel, prepare_contrasts, _flatten
-
+from .nistats import FirstLevelModel, _flatten, prepare_contrasts
 
 STAT_CODES = nb.volumeutils.Recoder(
     (
@@ -62,16 +61,16 @@ class FirstLevelModel(FirstLevelModel):
         """
         Fit a GLM using AFNI's 3dREMLfit
         """
-        from nipype import logging
         import nibabel as nb
-        from nipype.interfaces import afni
-
         import pandas as pd
+        from nipype import logging
+        from nipype.interfaces import afni
 
         logger = logging.getLogger("nipype.interface")
 
+        spec = self.inputs.spec
         mat = pd.read_csv(self.inputs.design_matrix, delimiter="\t", index_col=0)
-        contrasts = prepare_contrasts(self.inputs.contrast_info, mat.columns.tolist())
+        contrasts = prepare_contrasts(spec['contrasts'], mat.columns.tolist())
         t_r = mat.index[1]
         design_fname = op.join(runtime.cwd, "design.xmat.1D")
         stim_labels = self.get_stim_labels()
@@ -117,9 +116,7 @@ class FirstLevelModel(FirstLevelModel):
             img.to_filename(img_path)
 
         # Execute commands
-        logger.info(
-            f"3dREMLfit and 3dPval computation will be performed in: {runtime.cwd}\n"
-        )
+        logger.info(f"3dREMLfit and 3dPval computation will be performed in: {runtime.cwd}\n")
 
         # Define 3dREMLfit command
         remlfit = afni.Remlfit()
@@ -144,10 +141,9 @@ class FirstLevelModel(FirstLevelModel):
         fwhm.inputs.in_file = reml_res.outputs.wherr_file
         fwhm.inputs.out_file = fname_fmt("model", "residsmoothness").replace('.nii.gz', '.tsv')
         fwhm_res = fwhm.run()
-        fwhm_dat = pd.read_csv(fwhm_res.outputs.out_file,  delim_whitespace=True, header=None)
+        fwhm_dat = pd.read_csv(fwhm_res.outputs.out_file, delim_whitespace=True, header=None)
         fwhm_dat.to_csv(fwhm_res.outputs.out_file, index=None, header=False, sep='\t')
 
-        out_ents = self.inputs.contrast_info[0]["entities"]
         out_maps = nb.load(reml_res.outputs.out_file)
         var_maps = nb.load(reml_res.outputs.var_file)
         beta_maps = nb.load(reml_res.outputs.rbeta_file)
@@ -165,7 +161,7 @@ class FirstLevelModel(FirstLevelModel):
         model_maps = []
         model_metadata = []
         for attr, (imgs, idx) in model_attr_extract.items():
-            model_metadata.append({'stat': attr, **out_ents})
+            model_metadata.append({'stat': attr, **spec['entities']})
             fname = fname_fmt('model', attr)
             extract_volume(imgs, idx, f"{attr} of model", fname)
             model_maps.append(fname)
@@ -173,14 +169,14 @@ class FirstLevelModel(FirstLevelModel):
         # separate dict for maps that don't need to be extracted
         model_attr = {
             'residtsnr': self.save_tsnr(runtime, beta_maps, var_maps),
-            'residsmoothness': fwhm_res.outputs.out_file
+            'residsmoothness': fwhm_res.outputs.out_file,
         }
         # Save error time series if people want it
         if self.errorts:
             model_attr["errorts"] = reml_res.outputs.wherr_file
 
         for attr, fname in model_attr.items():
-            model_metadata.append({'stat': attr, **out_ents})
+            model_metadata.append({'stat': attr, **spec["entities"]})
             model_maps.append(fname)
 
         # get pvals and zscore buckets (niftis with heterogeneous intent codes)
@@ -240,8 +236,6 @@ class FirstLevelModel(FirstLevelModel):
         pvalue_maps = []
         fname_fmt = op.join(runtime.cwd, "{}_{}.nii.gz").format
 
-        out_ents = self.inputs.contrast_info[0]["entities"]
-
         stats_img_info = parse_afni_ext(maps["stat"])
 
         stat_types = np.array(
@@ -262,13 +256,18 @@ class FirstLevelModel(FirstLevelModel):
                 clean_vol_labels.append(x.rsplit('_', 1)[0])
             else:
                 clean_vol_labels.append(x)
-        for (name, weights, contrast_type) in contrasts:
+        for (name, weights, cont_ents, contrast_test) in contrasts:
             contrast_metadata.append(
-                {"contrast": name, "stat": contrast_type, **out_ents}
+                {
+                    "name": self.inputs.spec['name'],
+                    "level": self.inputs.spec['level'],
+                    "stat": contrast_test,
+                    **cont_ents,
+                }
             )
 
             # Get boolean to index appropriate values
-            stat_bool = stat_types == contrast_type.upper()
+            stat_bool = stat_types == contrast_test.upper()
             contrast_bool = np.array(clean_vol_labels) == name
 
             # Indices for multi image nibabel object  should have length 1 and be integers
@@ -297,21 +296,20 @@ class FirstLevelModel(FirstLevelModel):
                     imgs = maps[map_type]
                     fname = fname_fmt(name, map_type)
                     extract_volume(
-                        imgs,
-                        idx,
-                        f"{map_type} of contrast {name}",
-                        fname_fmt(name, map_type)
+                        imgs, idx, f"{map_type} of contrast {name}", fname_fmt(name, map_type)
                     )
                     map_list.append(fname)
 
         # calculate effect variance
-        for (name, weights, contrast_type), effect_fname, stat_fname in zip(contrasts, effect_maps, stat_maps):
+        for (name, weights, contrast_entities, contrast_type), effect_fname, stat_fname in zip(
+            contrasts, effect_maps, stat_maps
+        ):
             map_type = "effect_variance"
             effect_img = nb.load(effect_fname)
             effect = effect_img.get_fdata()
             stat_img = nb.load(stat_fname)
             stat = stat_img.get_fdata()
-            variance = ((effect/stat)) ** 2
+            variance = (effect / stat) ** 2
             variance_img = nb.Nifti1Image(variance, effect_img.affine, effect_img.header)
             variance_img.header['descrip'] = f"{map_type} of contrast {name}"
 
@@ -329,14 +327,19 @@ class FirstLevelModel(FirstLevelModel):
     def get_stim_labels(self):
         # Iterate through all weight specifications to get a list of stimulus
         # column labels.
-        weights = _flatten([x["weights"] for x in self.inputs.contrast_info])
-        return list(set(_flatten([x.keys() for x in weights])))
+        conditions = _flatten(
+            [contrast_info['conditions'] for contrast_info in self.inputs.spec['contrasts']]
+        )
+        return list(set(conditions))
 
     def save_tsnr(self, runtime, rbetas, rvars):
         vol_labels = parse_afni_ext(rbetas)["BRICK_LABS"].split("~")
         mat = pd.read_csv(self.inputs.design_matrix, delimiter="\t", index_col=0)
         # find the name of the constant column
-        const_name = mat.columns[(mat != 1).sum(0) == 0].values[0]
+        if 'constant' in mat.columns:
+            const_name = 'constant'
+        else:
+            const_name = mat.columns[np.isclose(mat, 1).all(0)].values[0]
         const_idx = np.where(np.array(vol_labels) == const_name)[0]
         const_dat = rbetas.slicer[..., int(const_idx)].get_fdata()
         std_img = rvars.slicer[..., 3]
@@ -383,7 +386,7 @@ def get_afni_design_matrix(design, contrasts, stim_labels, t_r):
     """
 
     cols = list(design.columns)
-    stim_col_nums = sorted([cols.index(x) for x in stim_labels])
+    stim_col_nums = sorted(cols.index(x) for x in stim_labels)
 
     # Currently multi-column stimuli not supported. If they were stim_tops
     # would need to be computed
@@ -419,7 +422,7 @@ def get_afni_design_matrix(design, contrasts, stim_labels, t_r):
 
 def create_glt_test_info(design, contrasts):
 
-    labels, wts_arrays, test_vals = zip(*contrasts)
+    labels, wts_arrays, cont_ents, test_vals = zip(*contrasts)
 
     # Start defining a list containing the rows for the glt values in the
     # afni design matrix header:
@@ -515,9 +518,7 @@ def get_afni_intent_info(img):
             intent_info.append(tuple(val))
         else:
             params = [x for x in val[1].split(",")]
-            intent_info.append(
-                (STAT_CODES.label[val[0]], tuple([float(x) for x in params if x]))
-            )
+            intent_info.append((STAT_CODES.label[val[0]], tuple(float(x) for x in params if x)))
 
     return intent_info
 
@@ -531,9 +532,7 @@ class PvalInputSpec(AFNICommandInputSpec):
         mandatory=True,
         exists=True,
     )
-    zscore = traits.Bool(
-        usedefault=False, argstr="-zscore", desc="convert to a z-score instead"
-    )
+    zscore = traits.Bool(usedefault=False, argstr="-zscore", desc="convert to a z-score instead")
     out_file = File(
         desc="Filename (AFNI prefix) for the output.",
         name_template="%s_stat",
@@ -584,9 +583,7 @@ class Pval(AFNICommand):
                 suffix = ""
         else:
             prefix = self.inputs.out_file
-            ext_ind = max(
-                [prefix.lower().rfind(".nii.gz"), prefix.lower().rfind(".nii")]
-            )
+            ext_ind = max([prefix.lower().rfind(".nii.gz"), prefix.lower().rfind(".nii")])
             if ext_ind == -1:
                 ext = ".HEAD"
                 suffix = "+tlrc"
@@ -629,7 +626,7 @@ def parse_afni_ext(nifti_file):
         vtype = type_mapping[attribute.attrib["ni_type"]]
         vname = attribute.attrib["atr_name"]
         vcount = attribute.attrib["ni_dimen"]
-        vval = attribute.text.strip('\n "').replace('"\n "','')
+        vval = attribute.text.strip('\n "').replace('"\n "', '')
         # Create a string object equivalent to what is observed when
         # parsing an AFNI ".HEAD" file.
         tmp = "type = {vtype}\nname = {vname}\ncount = {vcount}\n{vval}\n"
